aboutsummaryrefslogtreecommitdiff
path: root/block.c
diff options
context:
space:
mode:
Diffstat (limited to 'block.c')
-rw-r--r--block.c42
1 files changed, 32 insertions, 10 deletions
diff --git a/block.c b/block.c
index c139540..29e931e 100644
--- a/block.c
+++ b/block.c
@@ -2472,18 +2472,20 @@ void bdrv_root_unref_child(BdrvChild *child)
bdrv_unref(child_bs);
}
-void bdrv_unref_child(BlockDriverState *parent, BdrvChild *child)
+/**
+ * Clear all inherits_from pointers from children and grandchildren of
+ * @root that point to @root, where necessary.
+ */
+static void bdrv_unset_inherits_from(BlockDriverState *root, BdrvChild *child)
{
- if (child == NULL) {
- return;
- }
-
- if (child->bs->inherits_from == parent) {
- BdrvChild *c;
+ BdrvChild *c;
- /* Remove inherits_from only when the last reference between parent and
- * child->bs goes away. */
- QLIST_FOREACH(c, &parent->children, next) {
+ if (child->bs->inherits_from == root) {
+ /*
+ * Remove inherits_from only when the last reference between root and
+ * child->bs goes away.
+ */
+ QLIST_FOREACH(c, &root->children, next) {
if (c != child && c->bs == child->bs) {
break;
}
@@ -2493,6 +2495,18 @@ void bdrv_unref_child(BlockDriverState *parent, BdrvChild *child)
}
}
+ QLIST_FOREACH(c, &child->bs->children, next) {
+ bdrv_unset_inherits_from(root, c);
+ }
+}
+
+void bdrv_unref_child(BlockDriverState *parent, BdrvChild *child)
+{
+ if (child == NULL) {
+ return;
+ }
+
+ bdrv_unset_inherits_from(parent, child);
bdrv_root_unref_child(child);
}
@@ -4417,6 +4431,14 @@ int bdrv_freeze_backing_chain(BlockDriverState *bs, BlockDriverState *base,
}
for (i = bs; i != base; i = backing_bs(i)) {
+ if (i->backing && backing_bs(i)->never_freeze) {
+ error_setg(errp, "Cannot freeze '%s' link to '%s'",
+ i->backing->name, backing_bs(i)->node_name);
+ return -EPERM;
+ }
+ }
+
+ for (i = bs; i != base; i = backing_bs(i)) {
if (i->backing) {
i->backing->frozen = true;
}